package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.space.service.ISpaceOptimizeShortCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.*;
import pps.core.space.enums.BusinessError;
import pps.core.space.mapper.SpaceInstitutionWellheadMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 短期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/20 11:44
 */
@XService
public class SpaceOptimizeShortCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceOptimizeShortCloudService {

    /**
     * 短期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeShortJob(XContext context) {
        //取生效中的基础间开
        DateTime date = DateUtil.date();
        String curDate = date.toString(BusinessConstant.DATE_FORMAT_DAY);
        List<SpaceInstitutionDetailEnt> detailEntList = super.getEffectiveSpaceInstitution(context, curDate);
        List<String> institutionIdList = detailEntList.stream()
                .map(BaseModel::getId)
                .collect(Collectors.toList());
        //取制度关联的所有井口
        SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
        List<SpaceInstitutionWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .in(SpaceInstitutionWellheadEnt::getInstitutionId, institutionIdList)
        );
        if (CollUtil.isEmpty(wellheadList)) {
            //未发现可优化井口
            throw new XServiceException(BusinessError.DidNotFindWellhead);
        }
        Map<String, List<SpaceInstitutionWellheadEnt>> wellheadsByInstitutionIdIdMap = wellheadList.stream()
                .collect(Collectors.groupingBy(SpaceInstitutionWellheadEnt::getInstitutionId));
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        DateTime endOfDay = DateUtil.endOfDay(date);
        //优化
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            String detailId = detail.getId();
            String lineId = detail.getLineId();
            //创建记录
            String periodId = super.createOptimizePeriod(periodDTOList, detailId, lineId, null, endOfDay);
            if (!wellheadsByInstitutionIdIdMap.containsKey(detailId)) {
                continue;
            }
            List<SpaceInstitutionWellheadEnt> wellheadEntList = wellheadsByInstitutionIdIdMap.get(detailId);

        }
        return XServiceResult.OK;
    }
}